==> polar-0.2.3.tar.gz: lib/polar.dart <==
import 'dart:async';
import 'dart:convert';
import 'dart:io' show Platform;

import 'package:enum_to_string/enum_to_string.dart';
import 'package:flutter/services.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:polar/model/xyz.dart';
import 'package:recase/recase.dart';

part 'model/device_streaming_feature.dart';
part 'model/polar_device_info.dart';
part 'model/polar_hr_data.dart';
part 'model/polar_streaming.dart';
part 'model/ohr_data_type.dart';
part 'model/polar_ohr_ppi_sample.dart';
part 'model/polar_sensor_setting.dart';
part 'events.dart';

class Polar {
  static const MethodChannel _channel = const MethodChannel('polar');

  // Streaming
  final _ecgStreamController = StreamController<PolarEcgData>.broadcast();
  final _accStreamController = StreamController<PolarAccData>.broadcast();
  final _gyroStreamController = StreamController<PolarGyroData>.broadcast();
  final _magnetometerStreamController =
      StreamController<PolarMagnetometerData>.broadcast();
  final _ohrStreamController = StreamController<PolarOhrData>.broadcast();
  final _ohrPPIStreamController = StreamController<PolarPpiData>.broadcast();

  // Other data
  final _blePowerStateStreamController = StreamController<bool>.broadcast();
  final _deviceConnectedStreamController =
      StreamController<PolarDeviceInfo>.broadcast();
  final _deviceConnectingStreamController =
      StreamController<PolarDeviceInfo>.broadcast();
  final _deviceDisconnectedStreamController =
      StreamController<PolarDeviceInfo>.broadcast();
  final _streamingFeaturesReadyStreamController =
      StreamController<PolarStreamingFeaturesReadyEvent>.broadcast();
  final _sdkModeFeatureAvailableStreamController =
      StreamController<String>.broadcast();
  final _hrFeatureReadyStreamController = StreamController<String>.broadcast();
  final _disInformationStreamController =
      StreamController<PolarDisInformationEvent>.broadcast();
  final _batteryLevelStreamController =
      StreamController<PolarBatteryLevelEvent>.broadcast();
  final _heartRateStreamController =
      StreamController<PolarHeartRateEvent>.broadcast();
  final _ftpFeatureReadyStreamController = StreamController<String>.broadcast();

  Stream<bool> get blePowerStateStream => _blePowerStateStreamController.stream;
  Stream<PolarDeviceInfo> get deviceConnectedStream =>
      _deviceConnectedStreamController.stream;
  Stream<PolarDeviceInfo> get deviceConnectingStream =>
      _deviceConnectingStreamController.stream;
  Stream<PolarDeviceInfo> get deviceDisconnectedStream =>
      _deviceDisconnectedStreamController.stream;
  Stream<PolarStreamingFeaturesReadyEvent> get streamingFeaturesReadyStream =>
      _streamingFeaturesReadyStreamController.stream;
  Stream<String> get sdkModeFeatureAvailableStream =>
      _sdkModeFeatureAvailableStreamController.stream;
  Stream<String> get hrFeatureReadyStream =>
      _hrFeatureReadyStreamController.stream;
  Stream<PolarDisInformationEvent> get disInformationStream =>
      _disInformationStreamController.stream;
  Stream<PolarBatteryLevelEvent> get batteryLevelStream =>
      _batteryLevelStreamController.stream;
  Stream<PolarHeartRateEvent> get heartRateStream =>
      _heartRateStreamController.stream;
  Stream<String> get ftpFeatureReadyStream =>
      _ftpFeatureReadyStreamController.stream;

  /// Initialize the Polar API
  Polar() {
    _channel.setMethodCallHandler((call) {
      switch (call.method) {
        case 'ecgDataReceived':
          _ecgStreamController.add(PolarEcgData.fromJson(
            call.arguments[0],
            jsonDecode(call.arguments[1]),
          ));
          break;
        case 'accDataReceived':
          _accStreamController.add(PolarAccData.fromJson(
            call.arguments[0],
            jsonDecode(call.arguments[1]),
          ));
          break;
        case 'gyroDataReceived':
          _gyroStreamController.add(PolarGyroData.fromJson(
            call.arguments[0],
            jsonDecode(call.arguments[1]),
          ));
          break;
        case 'magnetometerDataReceived':
          _magnetometerStreamController.add(PolarMagnetometerData.fromJson(
            call.arguments[0],
            jsonDecode(call.arguments[1]),
          ));
          break;
        case 'ohrDataReceived':
          _ohrStreamController.add(PolarOhrData.fromJson(
            call.arguments[0],
            jsonDecode(call.arguments[1]),
          ));
          break;
        case 'ohrPPIReceived':
          _ohrPPIStreamController.add(PolarPpiData.fromJson(
            call.arguments[0],
            jsonDecode(call.arguments[1]),
          ));
          break;
        case 'blePowerStateChanged':
          _blePowerStateStreamController.add(call.arguments);
          break;
        case 'deviceConnected':
          _deviceConnectedStreamController
              .add(PolarDeviceInfo.fromJson(jsonDecode(call.arguments)));
          break;
        case 'deviceConnecting':
          _deviceConnectingStreamController
              .add(PolarDeviceInfo.fromJson(jsonDecode(call.arguments)));
          break;
        case 'deviceDisconnected':
          _deviceDisconnectedStreamController
              .add(PolarDeviceInfo.fromJson(jsonDecode(call.arguments)));
          break;
        case 'streamingFeaturesReady':
          _streamingFeaturesReadyStreamController.add(
            PolarStreamingFeaturesReadyEvent(
              call.arguments[0],
              (jsonDecode(call.arguments[1]) as List)
                  .map((e) => DeviceStreamingFeatureExtension.fromJson(e))
                  .toList(),
            ),
          );
          break;
        case 'sdkModeFeatureAvailable':
          _sdkModeFeatureAvailableStreamController.add(call.arguments);
          break;
        case 'hrFeatureReady':
          _hrFeatureReadyStreamController.add(call.arguments);
          break;
        case 'disInformationReceived':
          _disInformationStreamController.add(
            PolarDisInformationEvent(
              call.arguments[0],
              call.arguments[1],
              call.arguments[2],
            ),
          );
          break;
        case 'batteryLevelReceived':
          _batteryLevelStreamController.add(
            PolarBatteryLevelEvent(
              call.arguments[0],
              call.arguments[1],
            ),
          );
          break;
        case 'hrNotificationReceived':
          _heartRateStreamController.add(
            PolarHeartRateEvent(
              call.arguments[0],
              PolarHrData.fromJson(jsonDecode(call.arguments[1])),
            ),
          );
          break;
        case 'polarFtpFeatureReady':
          _ftpFeatureReadyStreamController.add(call.arguments);
          break;
      }

      return Future.value();
    });
  }

  /// Connect to a device with the given [identifier]
  void connectToDevice(String identifier) async {
    if (Platform.isAndroid) {
      await Permission.location.request();
    }

    _channel.invokeMethod('connectToDevice', identifier);
  }

  /// Disconnect from a device with the given [identifier]
  void disconnectFromDevice(String identifier) {
    _channel.invokeMethod('disconnectFromDevice', identifier);
  }

  Future<PolarSensorSetting> requestStreamSettings(
    String identifier,
    DeviceStreamingFeature feature,
  ) async {
    final response = await _channel.invokeMethod(
      'requestStreamSettings',
      [identifier, feature.toJson()],
    );
    final json = jsonDecode(response);
    return PolarSensorSetting.fromJson(json);
  }

  Stream<PolarEcgData> startEcgStreaming(
    String identifier, {
    PolarSensorSetting? settings,
  }) {
    _startEcgStreamingInternal(identifier, settings);
    return _ecgStreamController.stream.where((e) => e.identifier == identifier);
  }

  void _startEcgStreamingInternal(
    String identifier,
    PolarSensorSetting? settings,
  ) async {
    settings ??= await requestStreamSettings(
      identifier,
      DeviceStreamingFeature.ecg,
    );

    _channel.invokeMethod('startEcgStreaming', [
      identifier,
      jsonEncode(settings),
    ]);
  }

  Stream<PolarAccData> startAccStreaming(
    String identifier, {
    PolarSensorSetting? settings,
  }) {
    _startAccStreamingInternal(identifier);
    return _accStreamController.stream.where((e) => e.identifier == identifier);
  }

  void _startAccStreamingInternal(
    String identifier, {
    PolarSensorSetting? settings,
  }) async {
    settings ??= await requestStreamSettings(
      identifier,
      DeviceStreamingFeature.acc,
    );

    _channel.invokeMethod('startAccStreaming', [
      identifier,
      jsonEncode(settings),
    ]);
  }

  Stream<PolarGyroData> startGyroStreaming(
    String identifier, {
    PolarSensorSetting? settings,
  }) {
    _startGyroStreamingInternal(identifier, settings);
    return _gyroStreamController.stream
        .where((e) => e.identifier == identifier);
  }

  void _startGyroStreamingInternal(
    String identifier,
    PolarSensorSetting? settings,
  ) async {
    settings ??= await requestStreamSettings(
      identifier,
      DeviceStreamingFeature.gyro,
    );

    _channel.invokeMethod('startGyroStreaming', [
      identifier,
      jsonEncode(settings),
    ]);
  }

  Stream<PolarMagnetometerData> startMagnetometerStreaming(
    String identifier, {
    PolarSensorSetting? settings,
  }) {
    _startMagnetometerStreamingInternal(identifier, settings);
    return _magnetometerStreamController.stream
        .where((e) => e.identifier == identifier);
  }

  void _startMagnetometerStreamingInternal(
    String identifier,
    PolarSensorSetting? settings,
  ) async {
    settings ??= await requestStreamSettings(
      identifier,
      DeviceStreamingFeature.magnetometer,
    );

    _channel.invokeMethod('startMagnetometerStreaming', [
      identifier,
      jsonEncode(settings),
    ]);
  }

  Stream<PolarOhrData> startOhrStreaming(
    String identifier, {
    PolarSensorSetting? settings,
  }) {
    _startOhrStreamingInternal(identifier, settings);
    return _ohrStreamController.stream.where((e) => e.identifier == identifier);
  }

  void _startOhrStreamingInternal(
    String identifier,
    PolarSensorSetting? settings,
  ) async {
    settings ??= await requestStreamSettings(
      identifier,
      DeviceStreamingFeature.ppg,
    );

    _channel.invokeMethod('startOhrStreaming', [
      identifier,
      jsonEncode(settings),
    ]);
  }

  Stream<PolarPpiData> startOhrPPIStreaming(String identifier) {
    _channel.invokeMethod('startOhrPPIStreaming', identifier);
    return _ohrPPIStreamController.stream
        .where((e) => e.identifier == identifier);
  }
}

==> polar-1.0.1.tar.gz: lib/polar.dart <==
import 'dart:async';
import 'dart:convert';
import 'dart:io' show Platform;

import 'package:enum_to_string/enum_to_string.dart';
import 'package:flutter/services.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:polar/model/xyz.dart';
import 'package:recase/recase.dart';

part 'model/device_streaming_feature.dart';
part 'model/polar_device_info.dart';
part 'model/polar_hr_data.dart';
part 'model/polar_streaming.dart';
part 'model/ohr_data_type.dart';
part 'model/polar_ohr_ppi_sample.dart';
part 'model/polar_sensor_setting.dart';
part 'events.dart';

/// Flutter implementation of the [PolarBleSdk]
class Polar {
  static const MethodChannel _channel = const MethodChannel('polar');

  // Streaming
  final _ecgStreamController = StreamController<PolarEcgData>.broadcast();
  final _accStreamController = StreamController<PolarAccData>.broadcast();
  final _gyroStreamController = StreamController<PolarGyroData>.broadcast();
  final _magnetometerStreamController =
      StreamController<PolarMagnetometerData>.broadcast();
  final _ohrStreamController = StreamController<PolarOhrData>.broadcast();
  final _ohrPPIStreamController = StreamController<PolarPpiData>.broadcast();

  // Other data
  final _blePowerStateStreamController = StreamController<bool>.broadcast();
  final _deviceConnectedStreamController =
      StreamController<PolarDeviceInfo>.broadcast();
  final _deviceConnectingStreamController =
      StreamController<PolarDeviceInfo>.broadcast();
  final _deviceDisconnectedStreamController =
      StreamController<PolarDeviceInfo>.broadcast();
  final _streamingFeaturesReadyStreamController =
      StreamController<PolarStreamingFeaturesReadyEvent>.broadcast();
  final _sdkModeFeatureAvailableStreamController =
      StreamController<String>.broadcast();
  final _hrFeatureReadyStreamController = StreamController<String>.broadcast();
  final _disInformationStreamController =
      StreamController<PolarDisInformationEvent>.broadcast();
  final _batteryLevelStreamController =
      StreamController<PolarBatteryLevelEvent>.broadcast();
  final _heartRateStreamController =
      StreamController<PolarHeartRateEvent>.broadcast();
  final _ftpFeatureReadyStreamController = StreamController<String>.broadcast();

  /// helper to ask ble power state
  Stream<bool> get blePowerStateStream => _blePowerStateStreamController.stream;

  /// Device connnection has been established.
  ///
  /// - Parameter identifier: Polar device info
  Stream<PolarDeviceInfo> get deviceConnectedStream =>
      _deviceConnectedStreamController.stream;

  /// Callback when connection attempt is started to device
  ///
  /// - Parameter identifier: Polar device info
  Stream<PolarDeviceInfo> get deviceConnectingStream =>
      _deviceConnectingStreamController.stream;

  /// Connection lost to device.
  /// If PolarBleApi#disconnectFromPolarDevice is not called, a new connection attempt is dispatched automatically.
  ///
  /// - Parameter identifier: Polar device info
  Stream<PolarDeviceInfo> get deviceDisconnectedStream =>
      _deviceDisconnectedStreamController.stream;

  /// feature ready callback
  Stream<PolarStreamingFeaturesReadyEvent> get streamingFeaturesReadyStream =>
      _streamingFeaturesReadyStreamController.stream;

  /// sdk mode feature available in this device and ready for usage callback
  Stream<String> get sdkModeFeatureAvailableStream =>
      _sdkModeFeatureAvailableStreamController.stream;

  /// Device HR feature is ready. HR transmission is starting in a short while.
  ///
  /// - Parameter identifier: Polar device id
  Stream<String> get hrFeatureReadyStream =>
      _hrFeatureReadyStreamController.stream;

  ///  Received DIS info.
  ///
  /// - Parameters:
  ///   - identifier: Polar device id
  ///   - fwVersion: firmware version in format major.minor.patch
  Stream<PolarDisInformationEvent> get disInformationStream =>
      _disInformationStreamController.stream;

  /// Battery level received from device.
  ///
  /// - Parameters:
  ///   - identifier: Polar device id
  ///   - batteryLevel: battery level in precentage 0-100%
  Stream<PolarBatteryLevelEvent> get batteryLevelStream =>
      _batteryLevelStreamController.stream;

  /// HR notification received. Notice when using OH1 and PPI stream is started this callback will produce 0 hr.
  ///
  /// - Parameters:
  ///   - identifier: Polar device id
  Stream<PolarHeartRateEvent> get heartRateStream =>
      _heartRateStreamController.stream;

  /// Device file transfer protocol is ready.
  /// Notice all file transfer operations are preferred to be done at beginning of the connection
  ///
  /// - Parameter identifier: polar device id
  Stream<String> get ftpFeatureReadyStream =>
      _ftpFeatureReadyStreamController.stream;

  /// Initialize the Polar API.
  /// DartDocs are copied from the iOS version of the SDK and are only included for reference.
  Polar() {
    _channel.setMethodCallHandler((call) {
      switch (call.method) {
        case 'ecgDataReceived':
          _ecgStreamController.add(PolarEcgData.fromJson(
            call.arguments[0],
            jsonDecode(call.arguments[1]),
          ));
          break;
        case 'accDataReceived':
          _accStreamController.add(PolarAccData.fromJson(
            call.arguments[0],
            jsonDecode(call.arguments[1]),
          ));
          break;
        case 'gyroDataReceived':
          _gyroStreamController.add(PolarGyroData.fromJson(
            call.arguments[0],
            jsonDecode(call.arguments[1]),
          ));
          break;
        case 'magnetometerDataReceived':
          _magnetometerStreamController.add(PolarMagnetometerData.fromJson(
            call.arguments[0],
            jsonDecode(call.arguments[1]),
          ));
          break;
        case 'ohrDataReceived':
          _ohrStreamController.add(PolarOhrData.fromJson(
            call.arguments[0],
            jsonDecode(call.arguments[1]),
          ));
          break;
        case 'ohrPPIReceived':
          _ohrPPIStreamController.add(PolarPpiData.fromJson(
            call.arguments[0],
            jsonDecode(call.arguments[1]),
          ));
          break;
        case 'blePowerStateChanged':
          _blePowerStateStreamController.add(call.arguments);
          break;
        case 'deviceConnected':
          _deviceConnectedStreamController
              .add(PolarDeviceInfo.fromJson(jsonDecode(call.arguments)));
          break;
        case 'deviceConnecting':
          _deviceConnectingStreamController
              .add(PolarDeviceInfo.fromJson(jsonDecode(call.arguments)));
          break;
        case 'deviceDisconnected':
          _deviceDisconnectedStreamController
              .add(PolarDeviceInfo.fromJson(jsonDecode(call.arguments)));
          break;
        case 'streamingFeaturesReady':
          _streamingFeaturesReadyStreamController.add(
            PolarStreamingFeaturesReadyEvent(
              call.arguments[0],
              (jsonDecode(call.arguments[1]) as List)
                  .map((e) => DeviceStreamingFeatureExtension.fromJson(e))
                  .toList(),
            ),
          );
          break;
        case 'sdkModeFeatureAvailable':
          _sdkModeFeatureAvailableStreamController.add(call.arguments);
          break;
        case 'hrFeatureReady':
          _hrFeatureReadyStreamController.add(call.arguments);
          break;
        case 'disInformationReceived':
          _disInformationStreamController.add(
            PolarDisInformationEvent(
              call.arguments[0],
              call.arguments[1],
              call.arguments[2],
            ),
          );
          break;
        case 'batteryLevelReceived':
          _batteryLevelStreamController.add(
            PolarBatteryLevelEvent(
              call.arguments[0],
              call.arguments[1],
            ),
          );
          break;
        case 'hrNotificationReceived':
          _heartRateStreamController.add(
            PolarHeartRateEvent(
              call.arguments[0],
              PolarHrData.fromJson(jsonDecode(call.arguments[1])),
            ),
          );
          break;
        case 'polarFtpFeatureReady':
          _ftpFeatureReadyStreamController.add(call.arguments);
          break;
      }

      return Future.value();
    });
  }

  /// Request a connection to a Polar device. Invokes `PolarBleApiObservers` polarDeviceConnected.
  /// - Parameter identifier: Polar device id printed on the sensor/device or UUID.
  /// - Throws: InvalidArgument if identifier is invalid polar device id or invalid uuid
  void connectToDevice(String identifier) async {
    if (Platform.isAndroid) {
      await Permission.location.request();
    }

    _channel.invokeMethod('connectToDevice', identifier);
  }

  /// Disconnect from the current Polar device.
  ///
  /// - Parameter identifier: Polar device id
  /// - Throws: InvalidArgument if identifier is invalid polar device id or invalid uuid
  void disconnectFromDevice(String identifier) {
    _channel.invokeMethod('disconnectFromDevice', identifier);
  }

  ///  Request the stream settings available in current operation mode. This request shall be used before the stream is started
  ///  to decide currently available settings. The available settings depend on the state of the device. For example, if any stream(s)
  ///  or optical heart rate measurement is already enabled, then the device may limit the offer of possible settings for other stream feature.
  ///  Requires `polarSensorStreaming` feature.
  ///
  /// - Parameters:
  ///   - identifier: polar device id
  ///   - feature: selected feature from`DeviceStreamingFeature`
  /// - Returns: Single stream
  ///   - success: once after settings received from device
  ///   - onError: see `PolarErrors` for possible errors invoked
  Future<PolarSensorSetting> requestStreamSettings(
    String identifier,
    DeviceStreamingFeature feature,
  ) async {
    final response = await _channel.invokeMethod(
      'requestStreamSettings',
      [identifier, feature.toJson()],
    );
    final json = jsonDecode(response);
    return PolarSensorSetting.fromJson(json);
  }

  /// Start the ECG (Electrocardiography) stream. ECG stream is stopped if the connection is closed, error occurs or stream is disposed.
  /// Requires `polarSensorStreaming` feature. Before starting the stream it is recommended to query the available settings using `requestStreamSettings`
  ///
  /// - Parameters:
  ///   - identifier: Polar device id or device address
  ///   - settings: selected settings to start the stream
  /// - Returns: Observable stream
  ///   - onNext: for every air packet received. see `PolarEcgData`
  ///   - onError: see `PolarErrors` for possible errors invoked
  Stream<PolarEcgData> startEcgStreaming(
    String identifier, {
    PolarSensorSetting? settings,
  }) {
    _startEcgStreamingInternal(identifier, settings);
    return _ecgStreamController.stream.where((e) => e.identifier == identifier);
  }

  void _startEcgStreamingInternal(
    String identifier,
    PolarSensorSetting? settings,
  ) async {
    settings ??= await requestStreamSettings(
      identifier,
      DeviceStreamingFeature.ecg,
    );

    _channel.invokeMethod('startEcgStreaming', [
      identifier,
      jsonEncode(settings),
    ]);
  }

  ///  Start ACC (Accelerometer) stream. ACC stream is stopped if the connection is closed, error occurs or stream is disposed.
  ///  Requires `polarSensorStreaming` feature. Before starting the stream it is recommended to query the available settings using `requestStreamSettings`
  ///
  /// - Parameters:
  ///   - identifier: Polar device id or device address
  ///   - settings: selected settings to start the stream
  /// - Returns: Observable stream
  ///   - onNext: for every air packet received. see `PolarAccData`
  ///   - onError: see `PolarErrors` for possible errors invoked
  Stream<PolarAccData> startAccStreaming(
    String identifier, {
    PolarSensorSetting? settings,
  }) {
    _startAccStreamingInternal(identifier);
    return _accStreamController.stream.where((e) => e.identifier == identifier);
  }

  void _startAccStreamingInternal(
    String identifier, {
    PolarSensorSetting? settings,
  }) async {
    settings ??= await requestStreamSettings(
      identifier,
      DeviceStreamingFeature.acc,
    );

    _channel.invokeMethod('startAccStreaming', [
      identifier,
      jsonEncode(settings),
    ]);
  }

  /// Start Gyro stream. Gyro stream is stopped if the connection is closed, error occurs during start or stream is disposed.
  /// Requires `polarSensorStreaming` feature. Before starting the stream it is recommended to query the available settings using `requestStreamSettings`
  ///
  /// - Parameters:
  ///   - identifier: Polar device id or device address
  ///   - settings: selected settings to start the stream
  Stream<PolarGyroData> startGyroStreaming(
    String identifier, {
    PolarSensorSetting? settings,
  }) {
    _startGyroStreamingInternal(identifier, settings);
    return _gyroStreamController.stream
        .where((e) => e.identifier == identifier);
  }

  void _startGyroStreamingInternal(
    String identifier,
    PolarSensorSetting? settings,
  ) async {
    settings ??= await requestStreamSettings(
      identifier,
      DeviceStreamingFeature.gyro,
    );

    _channel.invokeMethod('startGyroStreaming', [
      identifier,
      jsonEncode(settings),
    ]);
  }

  /// Start magnetometer stream. Magnetometer stream is stopped if the connection is closed, error occurs or stream is disposed.
  /// Requires `polarSensorStreaming` feature. Before starting the stream it is recommended to query the available settings using `requestStreamSettings`
  ///
  /// - Parameters:
  ///   - identifier: Polar device id or device address
  ///   - settings: selected settings to start the stream
  Stream<PolarMagnetometerData> startMagnetometerStreaming(
    String identifier, {
    PolarSensorSetting? settings,
  }) {
    _startMagnetometerStreamingInternal(identifier, settings);
    return _magnetometerStreamController.stream
        .where((e) => e.identifier == identifier);
  }

  void _startMagnetometerStreamingInternal(
    String identifier,
    PolarSensorSetting? settings,
  ) async {
    settings ??= await requestStreamSettings(
      identifier,
      DeviceStreamingFeature.magnetometer,
    );

    _channel.invokeMethod('startMagnetometerStreaming', [
      identifier,
      jsonEncode(settings),
    ]);
  }

  /// Start OHR (Optical heart rate) PPG (Photoplethysmography) stream. PPG stream is stopped if the connection is closed, error occurs or stream is disposed.
  /// Requires `polarSensorStreaming` feature. Before starting the stream it is recommended to query the available settings using `requestStreamSettings`
  ///
  /// - Parameters:
  ///   - identifier: Polar device id or device address
  ///   - settings: selected settings to start the stream
  /// - Returns: Observable stream
  ///   - onNext: for every air packet received. see `PolarOhrData`
  ///   - onError: see `PolarErrors` for possible errors invoked
  Stream<PolarOhrData> startOhrStreaming(
    String identifier, {
    PolarSensorSetting? settings,
  }) {
    _startOhrStreamingInternal(identifier, settings);
    return _ohrStreamController.stream.where((e) => e.identifier == identifier);
  }

  void _startOhrStreamingInternal(
    String identifier,
    PolarSensorSetting? settings,
  ) async {
    settings ??= await requestStreamSettings(
      identifier,
      DeviceStreamingFeature.ppg,
    );

    _channel.invokeMethod('startOhrStreaming', [
      identifier,
      jsonEncode(settings),
    ]);
  }

  /// Start OHR (Optical heart rate) PPI (Pulse to Pulse interval) stream.
  /// PPI stream is stopped if the connection is closed, error occurs or stream is disposed.
  /// Notice that there is a delay before PPI data stream starts. Requires `polarSensorStreaming` feature.
  ///
  /// - Parameters:
  ///   - identifier: Polar device id or device address
  /// - Returns: Observable stream
  ///   - onNext: for every air packet received. see `PolarPpiData`
  ///   - onError: see `PolarErrors` for possible errors invoked
  Stream<PolarPpiData> startOhrPPIStreaming(String identifier) {
    _channel.invokeMethod('startOhrPPIStreaming', identifier);
    return _ohrPPIStreamController.stream
        .where((e) => e.identifier == identifier);
  }
}
